#include "gui/loot_ui.hpp"
#include "constants.hpp"
#include <fmt/xchar.h>

namespace gui {
  using namespace guecs;

  LootUI::LootUI(GameLevel level) :
      $level(level)
  {
    $gui.position(RAY_VIEW_X+RAY_VIEW_WIDTH/2-200,
        RAY_VIEW_Y+RAY_VIEW_HEIGHT/2-200, 400, 400);

    $gui.layout(
        "[item_0 | item_1 |item_2 | item_3 ]"
        "[item_4 | item_5 |item_6 | item_7 ]"
        "[item_8 | item_9 |item_10| item_11]"
        "[item_12| item_13|item_14|item_15 ]"
        "[_ | %(100,50)close| _]"
        );
  }

  void LootUI::init() {
    using guecs::THEME;
    auto bg_color = THEME.DARK_LIGHT;
    bg_color.a = 140;
    $gui.set<Background>($gui.MAIN, {$gui.$parser, bg_color});

    auto close = $gui.entity("close");
    $gui.set<guecs::Rectangle>(close, {});
    $gui.set<guecs::Label>(close, {L"CLOSE"});
    $gui.set<guecs::Clickable>(close,
        guecs::make_action($level, Events::GUI::LOOT_CLOSE));

    for(int i = 0; i < INV_SLOTS; i++) {
      auto id = $gui.entity("item_", i);
      $gui.set<guecs::Rectangle>(id, {THEME.PADDING,
          THEME.TRANSPARENT, THEME.LIGHT_MID });
      $gui.set<guecs::Effect>(id, {0.4f, "ui_shader"});
      $gui.set<guecs::Clickable>(id, {
          guecs::make_action($level, Events::GUI::LOOT_SELECT, {id})
      });
    }

    $gui.init();
    update();
  }

  void LootUI::update() {
    dbc::check(contents.size() < INV_SLOTS, "too many items in loot contents, must be < 16");

    for(size_t i = 0; i < INV_SLOTS; i++) {
      auto id = $gui.entity("item_", int(i));

      if(contents.contains(id)) {
        auto item = contents.at(id);
        dbc::check($level.world->has<components::Sprite>(item),
            "item in inventory UI doesn't exist in world. New level?");
        auto& sprite = $level.world->get<components::Sprite>(item);
        $gui.set_init<guecs::Sprite>(id, {sprite.name});

        guecs::GrabSource grabber{
            item, [&, id]() { return remove_slot(id); }};
        grabber.setSprite($gui, id);
        $gui.set<guecs::GrabSource>(id, grabber);
      } else {
        // BUG: fix remove so it's safe to call on empty
        if($gui.has<guecs::GrabSource>(id)) {
          $gui.remove<guecs::Sprite>(id);
          $gui.remove<guecs::GrabSource>(id);
        }

        $gui.set<guecs::DropTarget>(id, {
           [&, id](DinkyECS::Entity world_entity) -> bool { return place_slot(id, world_entity); }
        });
      }
    }
  }

  void LootUI::remove_slot(DinkyECS::Entity slot_id) {
    contents.erase(slot_id);
    update();
  }

  bool LootUI::place_slot(DinkyECS::Entity id, DinkyECS::Entity world_entity) {
    if(contents.size() < INV_SLOTS && !contents.contains(id)) {
      contents.try_emplace(id, world_entity);
      update();
      return true;
    } else {
      return false;
    }
  }

  void LootUI::render(sf::RenderWindow& window) {
    $gui.render(window);
  }

  void LootUI::update_level(GameLevel &level) {
    $level = level;
    contents.clear();
    init();
  }

  bool LootUI::mouse(float x, float y, bool hover) {
    return $gui.mouse(x, y, hover);
  }
}
