#include "gui/status_ui.hpp"
#include "components.hpp"
#include <guecs/ui.hpp>
#include "rand.hpp"
#include <fmt/xchar.h>
#include "gui/guecstra.hpp"

namespace gui {
  using namespace guecs;
  using std::any, std::any_cast, std::string, std::make_any;

  StatusUI::StatusUI(GameLevel level) :
    $level(level), $ritual_ui(level)
  {
    $gui.position(STATUS_UI_X, STATUS_UI_Y, STATUS_UI_WIDTH, STATUS_UI_HEIGHT);
    $gui.layout(
        "[ritual_ui]"
        "[earring|armor_head|amulet]"
        "[back|*%(200,300)character_view|_|armor_bdy]"
        "[hand_r|_|_                        |hand_l]"
        "[ring_r|_|_                        |ring_l]"
        "[pocket_r|armor_leg|pocket_l]");
  }

  void StatusUI::init() {
    $gui.set<Background>($gui.MAIN, {$gui.$parser});

    for(auto& [name, cell] : $gui.cells()) {
      if(name == "character_view") {
        auto char_view = $gui.entity(name);
        $gui.set<Rectangle>(char_view, {});
        $gui.set<Sprite>(char_view, {"armored_knight"});
      } else {
        auto button = $gui.entity(name);
        $gui.set<Rectangle>(button, {});
        $gui.set<ActionData>(button, {make_any<string>(name)});

        if(name == "ritual_ui") {
          $gui.set<Clickable>(button, {
              [this](auto, auto){ select_ritual(); }
          });
          $gui.set<Sound>(button, {"pickup"});
        } else {
          $gui.set<Textual>(button, {guecs::to_wstring(name)});
          $gui.set<Clickable>(button, {
            guecs::make_action($level, Events::GUI::INV_SELECT, {button})
          });
          $gui.set<DropTarget>(button, {
              .commit=[&, button](DinkyECS::Entity world_target) -> bool {
                return place_slot(button, world_target);
              }
          });
        }
      }
    }

    $ritual_ui.event(ritual::Event::STARTED);
    $gui.init();
  }

  bool StatusUI::mouse(float x, float y, bool hover) {
    if($ritual_ui.is_open()) {
      return $ritual_ui.mouse(x, y, hover);
    } else {
      return $gui.mouse(x, y, hover);
    }
  }

  void StatusUI::select_ritual() {
    $ritual_ui.event(ritual::Event::TOGGLE);
  }

  void StatusUI::update() {
    dbc::log("REWRITE ME!");
  }

  void StatusUI::render(sf::RenderWindow &window) {
    $gui.render(window);
    // $gui.debug_layout(window);
    $ritual_ui.render(window);
  }

  void StatusUI::update_level(GameLevel &level) {
    $level = level;
    init();
  }

  bool StatusUI::place_slot(DinkyECS::Entity gui_id, DinkyECS::Entity world_entity) {
    if($level.world->has<components::Sprite>(world_entity)) {
      auto& sprite = $level.world->get<components::Sprite>(world_entity);
      $gui.set_init<guecs::Sprite>(gui_id, {sprite.name});
      guecs::GrabSource grabber{ world_entity,
          [&, gui_id]() { return remove_slot(gui_id); }};
      grabber.setSprite($gui, gui_id);
      $gui.set<guecs::GrabSource>(gui_id, grabber);

      contents.insert_or_assign(gui_id, world_entity);
      return true;
    } else {
      return false;
    }
  }

  void StatusUI::remove_slot(DinkyECS::Entity slot_id) {
    if(contents.contains(slot_id)) {
      contents.erase(slot_id);
      $gui.remove<guecs::GrabSource>(slot_id);
      $gui.remove<guecs::Sprite>(slot_id);
    }
  }
}
